@@ -1,5 +1,5 @@
-export CC=gcc-9
-export CXX=g++-9
+export CC=clang
+export CXX=clang++
 export FC=gfortran-9
 export F77=gfortran-9
 export MPIRUN=/usr/local/bin/mpirun 
