@@ -56,22 +56,33 @@
 #include <queue>
 #include "array_resize.hpp"
 #include "HeapSort.hpp"
-
+namespace Fem2D {
+#include "R3.hpp"
+}
 template <class T>
-struct affectation: binary_function<T, T, T>
+struct affectation
 {
+	using first_argument_type  = T;
+	using second_argument_type = T;
+	using result_type          = T;
 	T& operator()(T& x, const T& y) const {return (x=y);}
 };
 
 template <class T>
-struct affectation_add: binary_function<T, T, T>
+struct affectation_add
 {
+	using first_argument_type  = T;
+	using second_argument_type = T;
+	using result_type          = T;
 	T& operator()(T& x, const T& y) const {return (x+=y);}// correct FH 25/10/2013
 };
 
 template <class T>
-struct affectation_sub: binary_function<T, T, T>
+struct affectation_sub
 {
+	using first_argument_type  = T;
+	using second_argument_type = T;
+	using result_type          = T;
 	T& operator()(T& x, const T& y) const {return (x-=y);}// correct FH 25/10/2013
 };
 
@@ -90,12 +101,18 @@ template<class T> inline T Square (const T &a){return a*a;}
 
 
 template<class K>
-struct Op2_dotproduct: public binary_function<Transpose<KN_<K> >,KN<K> *,K> {
+struct Op2_dotproduct {
+  using first_argument_type  = Transpose<KN_<K> >;
+  using second_argument_type = KN<K> *;
+  using result_type          = K;
   static K f( Transpose<KN_<K> > const & a, KN<K> * const& b)
    { return (conj(a.t),*b);} };
 
 template<class K>
-struct Op2_dotproduct_: public binary_function<Transpose<KN_<K> >,KN_<K> ,K> {
+struct Op2_dotproduct_ {
+  using first_argument_type  = Transpose<KN_<K> >;
+  using second_argument_type = KN_<K>;
+  using result_type          = K;
   static K f( Transpose<KN_<K> > const & a, KN_<K>  const& b)
    { return (conj(a.t),b);} };
 
@@ -929,6 +946,27 @@ void ArrayDCL()
     atype<KN_<K> >()->SetTypeLoop(atype<K*>(),atype<long*>());
     atype<KNM_<K> >()->SetTypeLoop(atype<K*>(),atype<long*>(),atype<long*>());
 
+      // map string  of array ..  Test FH Mai 2022 ...
+  
+    map_type[typeid(MyMap<String,K>*).name()] = new ForEachType<MyMap<String,K>*>(Initialize<MyMap<String,K> >,Delete<MyMap<String,K> >) ;
+
+    map_type_of_map[make_pair(atype<string*>(),atype<K>())]=atype<MyMap<String,K>*>();
+
+
+    typedef MyMap<String,KN<K> > MyMapofArray;
+    map_type[typeid(MyMapofArray*).name()] = new ForEachType<MyMapofArray*>(Initialize<MyMapofArray >,Delete<MyMapofArray >) ;
+    ffassert(TypeArray(atype<K>(),atype<string*>()));
+    // make_pair(atype<string*>(),atype<KN_<K> >())
+    //  real[string][int]
+    map_type_of_map[make_pair(TypeArray(atype<K>(),atype<string*>()),atype<long >())]=atype<MyMapofArray*>();
+    ;// map of   tableau
+  /*
+    atype<MyMap<String,K>*>()->Add("[","",new OneOperator2_<K*,MyMap<String,K>*,string*>(get_element<K>));
+    TheOperators->Add("&",new OneOperator2_<bool,MyMap<String,K>*,string*>(exist_element<K>));
+   TheOperators->Add("<<",new OneBinaryOperator<PrintP<MyMap<String,K>*> >);
+   Add<MyMap<String,K>* >("n",".",new OneOperator1<long,MyMap<String,K> *>(get_MyMap_n));
+   */
+
 }
 
 
@@ -938,7 +976,10 @@ template<class A,class B> pair<A,B> * pBuild(const A & a,const B & b)
 
 // add mars 2006
 template<class K,class L,class OP>
-struct set_A_BI: public binary_function<KN_<K>,pair<KN_<K>, KN_<L> > *,KN_<K> > {
+struct set_A_BI {
+  using first_argument_type  = KN_<K>;
+  using second_argument_type = pair<KN_<K>, KN_<L> > *;
+  using result_type          = KN_<K>;
   static KN_<K> f(const KN_<K>   & a, pair<KN_<K>, KN_<L> > * const & b)  {
     KN_<K> x(a);
     OP op;
@@ -966,7 +1007,10 @@ struct set_A_BI: public binary_function<KN_<K>,pair<KN_<K>, KN_<L> > *,KN_<K> >
 };
 //  add oct 2019  To:  real[int] b = a(I); // where a and I is also a array..
 template<class K,class L,class OP>
-struct init_A_BI: public binary_function<KN<K>* ,pair<KN_<K>, KN_<L> > *,KN<K>* > {
+struct init_A_BI {
+  using first_argument_type  = KN<K>*;
+  using second_argument_type = pair<KN_<K>, KN_<L> > *;
+  using result_type          = KN<K>*;
     static KN<K>* f( KN<K>  * const  & a, pair<KN_<K>, KN_<L> > * const & b)  {
         KN<K> * px(a);
         OP op;
@@ -995,7 +1039,10 @@ struct init_A_BI: public binary_function<KN<K>* ,pair<KN_<K>, KN_<L> > *,KN<K>*
     }
 };
 template<class K,class L,class OP>
-struct set_AI_B: public binary_function<pair<KN_<K>, KN_<L> > * ,KN_<K>, NothingType > {
+struct set_AI_B {
+  using first_argument_type  = pair<KN_<K>, KN_<L> > *;
+  using second_argument_type = KN_<K>;
+  using result_type          = NothingType;
   static NothingType  f( pair<KN_<K>, KN_<L> > * const & b,const KN_<K>   & a)  {
     KN_<K> x(a);
     OP op;
@@ -1047,7 +1094,10 @@ struct Op3_pacc: public ternary_function<KN_<K>,K,K,if_arth_KN_<K>*> {
 };
 template<class K> KNM_<K> Transp(KNM_<K>  M){ return M.t();} // Add FH July 2015
 template<class K>
-struct SetArray2: public binary_function<K,K,SetArray<K> > {
+struct SetArray2{
+  using first_argument_type  = K;
+  using second_argument_type = K;
+  using result_type          = SetArray<K>;
   static SetArray<K> f(const K & a,const K & b)  {
     // cout << "SubArray: " << a << " " << b << endl;
     //     SetArray(long nn,R oo=R(),R sstep=R(1)): o(oo),n(nn),step(sstep) {}
@@ -1342,6 +1392,7 @@ KNM<K> * set_Eye(KNM<K> *pA,const  Eye eye)
     return  pA;
 }
 extern aType aaaa_knlp;
+
 template<class K,class Z>
 void ArrayOperator()
 {
@@ -1453,6 +1504,9 @@ void ArrayOperator()
 
 
      TheOperators->Add("<-",
+ //      new OneOperator2_<KN<K> *,KN<K> *,R3>(&set_initR3),
+ //      new OneOperator2_<KN<K> *,KN<K> *,R3*>(&set_initR3),
+
        new OneOperator2_<KN<K> *,KN<K> *,Z>(&set_init),
        new InitArrayfromArray<K,KN<K>*,true>
     //   new OneOperator2_<KN<K> *,KN<K> *,KN<K> >(&set_init),
@@ -1825,11 +1879,6 @@ void ArrayOperator()
         new OneBinaryOperator<Op_ReadKN<K> >,
         new OneBinaryOperator<Op_ReadKNM<K> >
       );
-
-     map_type[typeid(MyMap<String,K>*).name()] = new ForEachType<MyMap<String,K>*>(Initialize<MyMap<String,K> >,Delete<MyMap<String,K> >) ;
-
-     map_type_of_map[make_pair(atype<string*>(),atype<K>())]=atype<MyMap<String,K>*>();
-
      atype<MyMap<String,K>*>()->Add("[","",new OneOperator2_<K*,MyMap<String,K>*,string*>(get_element<K>));
      TheOperators->Add("&",new OneOperator2_<bool,MyMap<String,K>*,string*>(exist_element<K>));
     TheOperators->Add("<<",new OneBinaryOperator<PrintP<MyMap<String,K>*> >);
